# Continious Integration of VDC

## Introduction

With full cloud automation anything can go wrong, mainly:

- A new OS release, or the update of a critical package
- A new ACS release
- A new Ansible release, or the update of a dependency plugin

In order to release cloud environment with confidence the following is advised:

- Deliver automation tools as "sealed" as possible. VDC is currently delivered as a docker image that includes playbooks, plugins, ansible, etc. 
- Test each VDC release against multiple OS versions, ACS versions, etc.

## How the testing works

For each new VDC release, or an update of its dependencies a CI release should be run.

The process involes to spawn a new ACS platform and test against it.

The whole process is orchestrated as a gitlab pipeline, and the ACS platform is deployed inside a Vagrant cluster.

It is possible to spawn several environments at the same time, but care with nomenclature is advised. 

## Requirements

The process has the following dependencies:

- The docker gitlab runner is the main coordinator
- Most build stages require to be run on the same host, because they leave local object such as VMs on Vagrant. Tags can be used to ensure that those stages run on the same host. Also an issue about "sticky" builds has been created with gitlab.
- The docker gitlab runner must run on "metal", because ACS hosts will be modelled as VMs, and they will run test VMs inside (KVM nested virtualization).
- Ansible and Vagrant will be run as steps in the Gitlab process, for that to happen they have been "dokerized".
- KVM, Libvirt need to be installed on the Metal host, with the right settings (nested kvm enabled)
- gitlab runner docker needs to be configured to mount the docker and libvirt sockets so that docker and libvirt actions can be performed as part of the building pipeline.
- The metal host requires IP connectivity to the Vagrant network (which is provided by default)
- A docker network needs to be setup on the same VLAN as the Vagrant management network, and IP space divided between Vagrant and Docker, so that gitlab stages can trigger ansible commands on Vagrant VMs
- A public VLAN will be modelled in ACS, and the metal host needs to have an interface and IP on this network so that Ansible tests can be triggered on VMs spawned withing the Test ACS (those are the nested VMs, inside the Ansible VMs that represent the ACS hosts)

# Future enhangements

This process can be used to black box test new ACS releases if a stage is included in which "edge" ACS packages are built.